class Album
  include Mongoid::Document
  include Mongoid::Timestamps

  include OrbitCategory::Categorizable
  include OrbitTag::Taggable
  include Slug
  field :name, as: :slug_title, localize: true
  field :description, localize: true
  field :cover, default: "default"
  field :cover_path   #can  refact
  field :tag_names
  field :uid, type: String
  field :rss2_id, type: String
  field :order, type: Integer, default: -1
  field :min_order, type: Integer, default: -1
  field :resize_gravity
  field :thumb_process_option, type: Integer, default: 0
	# has_and_belongs_to_many :tags, :class_name => "GalleryTag"
  has_many :album_images, :autosave => true, :dependent => :destroy
  has_many :album_colors, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :album_images, :allow_destroy => true
  accepts_nested_attributes_for :album_colors, :allow_destroy => true
  index({"order" => 1}, { unique: false, background: false })
  before_create do |record|
    topest_order = AlbumSetting.pluck(:min_order).first.to_i
    if topest_order.nil? || topest_order == 0
      tmp_order = -2
    else
      tmp_order = topest_order - 1
    end
    record.order = tmp_order
    AlbumSetting.update_all(:min_order=>tmp_order)
    true
  end
  after_destroy do
    min_order = AlbumSetting.pluck(:min_order).first
    if self.order == min_order
      AlbumSetting.update_all(:min_order=>min_order + 1)
    end
  end
  def resize_gravity
    """
    NorthWestGravity
      Position object at top-left of region
    NorthGravity
      Position object at top-center of region
    NorthEastGravity
      Position object at top-right of region
    WestGravity
      Position object at left-center of region
    CenterGravity
      Position object at center of region
    EastGravity
      Position object at right-center of region
    SouthWestGravity
      Position object at left-bottom of region
    SouthGravity
      Position object at bottom-center of region
    SouthEastGravity
      Position object at bottom-right of region
    """
    tmp = self[:resize_gravity]
    tmp.blank? ? OrbitHelper::SharedHash['gallery'][:resize_gravity]  : tmp
  end
  def self.find_by_param(input)
    self.find_by(uid: input.to_s)
  end
  def self.filter_by_tags(tags=[])
    tags = OrbitHelper.page_tags if tags.blank?
    tags = [tags].flatten.uniq
    if !(tags.include?("all"))
        tag_temp = Tag.all.select{|value| tags.include? value.id.to_s}
        tag_temp_length = 0
        tag_temp.each do |value| 
          if value.name==I18n.t('gallery.show_desc') || value.name==I18n.t('gallery.not_show_desc')
            tag_temp_length+=1
          end
        end
        if tag_temp_length!=0
            if (tags.length - tag_temp_length) == 0
               tags = ['all'] 
            end
        end
    end
    if tags.blank? || (tags.include?("all") rescue false)
      self.all
    else
      tags
      taggings = Tagging.where(:tag_id.in=>tags).map{|item| item.taggable_id}
      self.where(:id.in=>taggings)
    end
  end
  def cover
    tmp = self['cover']
    if tmp=='default'
      tmp = (self.album_images.first.id.to_s rescue 'default')
    end
    tmp
  end
  def cover_path(thumb_version='thumb')
    tmp = self['cover_path']
    if tmp.nil? && self.album_images.first
      case thumb_version
      when 'thumb'
        tmp = self.album_images.first.file.thumb.url
      when 'thumb_large'
        tmp = self.album_images.first.file.thumb_large.url
      when 'mobile'
        tmp = self.album_images.first.file.mobile.url
      end
    end
    tmp
  end
end